"""Cowcerts certificate render entry point."""
import pkgutil
from http.server import BaseHTTPRequestHandler, HTTPServer
import json
import os
import sys

from jinja2 import Template
from bs4 import BeautifulSoup
import htmlmin
from inlinestyler.utils import inline_css

CURRENT_MODULE = "cowcerts_certificate_render"

DATA_DIR = "data"

TEMPLATES_DIR = os.path.join(
    DATA_DIR,
    "templates"
)

CERTIFICATES_DIR = os.path.join(
    DATA_DIR,
    "certificates"
)

DEFAULT_TEMPLATE_FILE = os.path.join(
    TEMPLATES_DIR,
    "cowcerts-for-education.html"
)

DEFAULT_CERTIFICATE_FILE = os.path.join(
    CERTIFICATES_DIR,
    "cowcerts-edu-20190403.json"
)

SERVER_HOST = ""

SERVER_PORT = 8081


class BasicHTTPRequestHandler(BaseHTTPRequestHandler):

    def _simple_response(self):
        pass

    def _set_headers(self):
        self.send_response(200)
        self.send_header('Content-Type', 'text/html; charset=utf-8')
        self.end_headers()

    def do_GET(self):
        self._set_headers()
        self.wfile.write(self._simple_response())


def main():
    html = read_html_template()
    certificate = read_certificate()
    render = render_html(html, certificate)
    if len(sys.argv) > 1 and sys.argv[1] in ("-s", "--serve"):
        render_beautified = BeautifulSoup(
            render,
            features="html.parser"
        ).prettify()
        request_handler = type(
            "GeneratedHTTPRequestHandler",
            (BasicHTTPRequestHandler,),
            {
                "_simple_response": lambda s: render_beautified.encode('utf-8')
            }
        )
        print("Serving certificate at \"%s\":%d" % (SERVER_HOST, SERVER_PORT))
        HTTPServer((SERVER_HOST, SERVER_PORT), request_handler).serve_forever()
    else:
        render_inlined = inline_css(render)
        render_body = ''.join(
            '%s' % l for l in BeautifulSoup(
                render_inlined,
                features="lxml"
            ).body.contents)
        render_minified = htmlmin.minify(
            render_body, remove_empty_space=True)
        certificate["displayHtml"] = render_minified
        print(json.dumps(certificate))


def read_html_template(template_file = DEFAULT_TEMPLATE_FILE):
    return load_file(template_file)


def load_file(filename):
    return pkgutil.get_data(CURRENT_MODULE, filename).decode("utf-8")


def read_certificate(certificate_file = DEFAULT_CERTIFICATE_FILE):
    return json.loads(load_file(certificate_file))


def render_html(html, context):
    return Template(html).render(context)


if __name__ == "__main__":
    main()



"""A setuptools based setup module.

See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from os import path, walk
from io import open

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


# Data files
# https://stackoverflow.com/a/36693250
def package_files(directory):
    paths = []
    for (current_path, directories, filenames) in walk(directory):
        for filename in filenames:
            paths.append(path.join('..', current_path, filename))
    return paths

# Pipenv dependencies
def find_pipenv_dependencies():
    import pipfile
    return list(map(lambda x: x, pipfile.load().data['default']))


setup(
    name='cowcerts-certificate-render',
    version='0.0.7',
    description='HTML template render for Cowcerts certificates visualization',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://gitlab.com/btcalabs/certificate-render',
    author='BTCA Labs',
    author_email='btcalabs+cowcerts+certificate-render@btcassessors.com',
    classifiers=[  # Optional
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='cowcerts certificate template html render',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=["jinja2", "beautifulsoup4", "htmlmin", "inlinestyler"],
    # extras_require={  # Optional
    #     'dev': ['check-manifest'],
    #     'test': ['coverage'],
    # },
    package_data={
        'cowcerts_certificate_render': package_files(
            path.join("cowcerts_certificate_render", "data")),
    },
    entry_points={
        'console_scripts': [
            'cowcerts-certificate-render=cowcerts_certificate_render.__main__:main',
        ],
    },
    project_urls={
        'Bug Reports': 'https://gitlab.com/btcalabs/certificate-render/issues',
        'Source': 'https://gitlab.com/btcalabs/certificate-render',
    },
)
